import sys
from os import path, getcwd
import yaml
import argparse
from bossman.errors import BossmanError
from bossman.cli import status_cmd, log_cmd, apply_cmd, validate_cmd, release_cmd, init_cmd
from bossman.config import Config
from bossman.logging import logger
from bossman import Bossman
from bossman.config import Config
from rich import print

# Pretty tracebacks
from rich.traceback import install
install()

logger = logger.getChild(globals().get("__name__"))

def main():
  try:
    parser = argparse.ArgumentParser()
    parser.add_argument("--verbosity", action=SetVerbosity, default="ERROR", type=str, choices=["DEBUG", "INFO", "WARNING", "ERROR"])
    parser.add_argument("--repo", help="path to the repository", default=getcwd())

    subparsers = parser.add_subparsers(title="Subcommands")
    version_cmd_init(subparsers)
    init_cmd.init(subparsers)
    status_cmd.init(subparsers)
    log_cmd.init(subparsers)
    apply_cmd.init(subparsers)
    validate_cmd.init(subparsers)
    release_cmd.init(subparsers)

    args = parser.parse_args()
    bossman = create_bossman(args)

    if "func" in args:
      args.func(bossman, **vars(args))
    else:
      parser.print_usage()
  except BossmanError as err:
    print(err)
    sys.exit(1)
  except KeyboardInterrupt:
    print("\ninterrupted by the user", file=sys.stderr)
    sys.exit(2)

def version_cmd_init(subparsers: argparse._SubParsersAction):
  parser = subparsers.add_parser("version", help="print the current version of bossman")
  parser.set_defaults(func=lambda bossman, *args, **kwargs: print(bossman.version))

def create_bossman(args):
  conf_path = path.join(args.repo, ".bossman")
  conf_data = {}
  if path.exists(conf_path):
    fd = open(conf_path, "r")
    conf_data = yaml.safe_load(fd)

  config = Config(conf_data)

  bossman = Bossman(args.repo, config)
  return bossman

class SetVerbosity(argparse.Action):
  """
  Set logger verbosity when --verbosity argument is provided.
  """
  def __init__(self, *args, **kwargs):
    super(SetVerbosity, self).__init__(*args, **kwargs)

  def __call__(self, parser, namespace, values, option_string=None):
    import logging
    logging.basicConfig(level=values)
    logger.info("SetVerbosity verbosity={verbosity}".format(verbosity=values))

if __name__ == '__main__':
  # Make bossman.cli.__init__ callable (for VS Code Debug)
  main()


from datetime import datetime
import dateutil.parser
from typing import List
from cattr import Converter

from bossman.logging import get_class_logger
from bossman.plugins.akamai.lib.edgegrid import Session
from .data import *
from .error import *

class CloudletAPIV3Client:
  def __init__(self, edgerc, section, switch_key=None, **kwargs):
    self.logger = get_class_logger(self)
    self.session = Session(edgerc, section, switch_key=switch_key, **kwargs)
    self.converter = Converter()
    self.converter.register_structure_hook(datetime, lambda d, t: dateutil.parser.isoparse(d))

  def get_policy_by_name(self, name):
    self.logger.debug("get_policy_by_name name={name}".format(name=name))
    id = None
    policies = self.get_policies()
    for policy in policies:
      if policy.name == name:
        return policy
    raise PolicyNameNotFound(name)

  def get_policies(self) -> List[SharedPolicy]:
    self.logger.debug("get_policies")
    response = self.session.get("/cloudlets/v3/policies")
    if response.status_code != 200:
      raise CloudletAPIV3Error(response.json())
    # import json
    # print(json.dumps(response.json(), indent=2))
    response = self.converter.structure(response.json(), GetPoliciesResponse)
    return response.content

  def get_policy_version(self, policyId: int, policyVersion: int) -> SharedPolicyVersion:
    self.logger.debug("get_policy_version policyId={policyId}, policyVersion={policyVersion}".format(policyId=policyId, policyVersion=policyVersion))
    response = self.session.get("/cloudlets/v3/policies/{policyId}/versions/{policyVersion}".format(policyId=policyId, policyVersion=policyVersion))
    if response.status_code != 200:
      raise CloudletAPIV3Error(response.json())
    return self.converter.structure(response.json(), SharedPolicyVersion)

  def get_latest_policy_version(self, policyId: int) -> Optional[SharedPolicyVersion]:
    self.logger.debug("get_policy_version policyId={policyId}".format(policyId=policyId))
    response = self.session.get("/cloudlets/v3/policies/{policyId}/versions".format(policyId=policyId), params=dict(size=10))
    if response.status_code != 200:
      raise CloudletAPIV3Error(response.json())
    policy_versions = self.converter.structure(response.json(), GetPolicyVersionsResponse)
    if len(policy_versions.content):
      return policy_versions.content[0]
    return None

  def get_latest_policy_versions(self, policyId: int, count: int) -> List[SharedPolicyVersion]:
    self.logger.debug("get_latest_policy_versions policyId={policyId}, count={count}".format(policyId=policyId, count=count))
    response = self.session.get("/cloudlets/v3/policies/{policyId}/versions".format(policyId=policyId), params=dict(size=count))
    if response.status_code != 200:
      raise CloudletAPIV3Error(response.json())
    policy_versions = self.converter.structure(response.json(), GetPolicyVersionsResponse)
    return policy_versions.content

  def create_policy(self, policyName: str, description: str, cloudletType: SharedPolicyCloudletType, groupId: int) -> SharedPolicy:
    self.logger.debug("create_policy policyName={policyName}".format(policyName=policyName))
    response = self.session.post("/cloudlets/v3/policies", json=dict(
      name=policyName,
      cloudletType=cloudletType.value,
      groupId=groupId,
      description=description,
    ))
    if response.status_code != 201:
      raise CloudletAPIV3Error(response.json())
    return self.converter.structure(response.json(), SharedPolicy)

  def update_policy(self, policyId: int, description: str, groupId: int) -> SharedPolicy:
    self.logger.debug("update_policy policyName={policyId}".format(policyId=policyId))
    response = self.session.put("/cloudlets/v3/policies/{policyId}".format(policyId=policyId), json=dict(
      groupId=groupId,
      description=description,
    ))
    if response.status_code != 202:
      raise CloudletAPIV3Error(response.json())
    return self.converter.structure(response.json(), SharedPolicy)

  def create_policy_version(self, policyId: int, description: str, matchRules: List[dict]) -> SharedPolicyVersion:
    self.logger.debug("create_policy_version policyId={policyId}".format(policyId=policyId))
    response = self.session.post("/cloudlets/v3/policies/{policyId}/versions".format(policyId=policyId), json=dict(
      description=description,
      matchRules=matchRules,
    ))
    if response.status_code != 201:
      raise CloudletAPIV3Error(response.json())
    return self.converter.structure(response.json(), SharedPolicyVersion)

  def activate_policy_version(self, policyId: int, policyVersion: int, network: Network) -> SharedPolicyActivation:
    self.logger.debug("activate_policy_version policyId={policyId}, policyVersion={policyVersion}".format(policyId=policyId, policyVersion=policyVersion))
    response = self.session.post("/cloudlets/v3/policies/{policyId}/activations".format(policyId=policyId), json=dict(
      network=network.value,
      operation=SharedPolicyActivationOperation.ACTIVATION.value,
      policyVersion=policyVersion,
    ))
    if response.status_code != 202:
      result = response.json().get('errors')[0]
      raise CloudletAPIV3Error(result.get('detail'))
    return self.converter.structure(response.json(), SharedPolicyActivation)

  def get_policy_version_activation_status(self, activation: SharedPolicyActivation) -> SharedPolicyActivation:
    self.logger.debug("get_policy_version_activation_status policyId={policyId}, activationId={activationId}".format(policyId=activation.policyId, activationId=activation.id))
    response = self.session.get("/cloudlets/v3/policies/{policyId}/activations/{activationId}".format(policyId=activation.policyId, activationId=activation.id))
    if response.status_code != 200:
      raise CloudletAPIV3Error(response.json())
    return self.converter.structure(response.json(), SharedPolicyActivation)



import sys, os
import requests
from requests.adapters import HTTPAdapter, RetryError
from urllib3.util.retry import Retry, MaxRetryError
from akamai.edgegrid import EdgeGridAuth, EdgeRc
from configparser import NoSectionError, NoOptionError
from bossman.logging import logger
from bossman.errors import BossmanError

py3 = sys.version_info[0] >= 3
if py3:
  from configparser import ConfigParser
  from urllib import parse
else:
  import ConfigParser
  # pylint: disable=import-error
  import urlparse as parse

class EdgegridError(BossmanError):
  pass

adapter = HTTPAdapter(
  pool_connections=1,
  pool_maxsize=10,
  max_retries=Retry(
    total=3,
    backoff_factor=5,
    status_forcelist=[429, 500, 502, 503, 504],
    method_whitelist=["HEAD", "GET", "PUT", "DELETE", "OPTIONS", "TRACE"],
  )
)

class Session(requests.Session):
  def __init__(self, edgerc, section, switch_key=None, **kwargs):
    try:
      super(Session, self).__init__(**kwargs)
      self.edgerc = EdgeRc(edgerc)
      self.section = section
      self.switch_key = switch_key
      self.auth = EdgeGridAuth(
        client_token=self.edgerc.get(section, "client_token"),
        client_secret=self.edgerc.get(section, "client_secret"),
        access_token=self.edgerc.get(section, "access_token"),
      )
      self.mount("https://", adapter)
      self.mount("http://", adapter)
    except NoSectionError as e:
      raise EdgegridError(e.message)
    except NoOptionError as e:
      raise EdgegridError(e.message)

  def request(self, method, url, params=None, **kwargs):
    try:
      if self.switch_key:
        params = params if params else {}
        params.update(accountSwitchKey=self.switch_key)
      baseUrl = "https://{host}".format(host=self.edgerc.get(self.section, "host"))
      url = parse.urljoin(baseUrl, url)
      response = super(Session, self).request(method, url, params=params, **kwargs)
      if response.status_code in (401, 403, *range(500, 600)):
        raise EdgegridError(response.json())
      return response
    except RetryError as e:
      raise EdgegridError(str(e))



from bossman.errors import BossmanError

class CloudletAPIV3Error(BossmanError):
  pass

class PolicyNameNotFound(CloudletAPIV3Error):
  pass

class PolicyVersionValidationError(CloudletAPIV3Error):
  pass
